;  ------------------------------------------------------------------

;  These procedures are common to both calculator procedures.

;  ------------------------------------------------------------------

Proc Ten_Key_Init()

;  General initialization for the 10-key calculator.

;  Initialize the desktop

Echo Off
Clear
Reset
AltSpace {Desktop} {Empty}         ;close all windows

;  Set up the "event dispatch" dynarrays, which convert events to actions

DynArray do_acts[]
DynArray do_objs[]
DynArray do_defs[]

;  Actions in response to special function keys

do_acts["Key->Ascii 27"]  = "Quit"              ;[Esc] key

;  Actions in response to menu events

do_acts["Message->MenuSelect->Quit"]               = "Quit"
do_acts["Message->MenuSelect->ClearAll"]           = "Proc"
do_objs["Message->MenuSelect->ClearAll"]           = "Double_Clear"
do_acts["Message->MenuSelect->EventTraceOn"]       = "Proc"
do_objs["Message->MenuSelect->EventTraceOn"]       = "Event_Trace_On"
do_acts["Message->MenuSelect->EventTraceOff"]      = "Proc"
do_objs["Message->MenuSelect->EventTraceOff"]      = "Event_Trace_Off"
do_acts["Message->MenuSelect->EventTraceConfig"]   = "Proc"
do_objs["Message->MenuSelect->EventTraceConfig"]   = "Event_Trace_Config"

;  Events which require further information to be added to them

do_acts["Mouse"]                 = "AddEvElem"
do_objs["Mouse"]                 = "Buttons"
do_defs["Mouse"]                 = "ExecEvent"
do_acts["Mouse->Left"]           = "AddEvElem"
do_objs["Mouse->Left"]           = "DoubleClick"
do_acts["Mouse->Left->False"]    = "AddEvElem"
do_objs["Mouse->Left->False"]    = "Action"
do_acts["Mouse->Right"]          = "AddEvElem"
do_objs["Mouse->Right"]          = "DoubleClick"
do_acts["Message"]               = "AddEvElem"
do_objs["Message"]               = "Message"
do_defs["Message"]               = "ExecEvent"
do_acts["Message->MenuSelect"]   = "AddEvElem"
do_objs["Message->MenuSelect"]   = "MenuTag"
do_acts["Message->Close"]        = "AddWindow"
do_acts["Key"]                   = "AddEvKey"
do_defs["Key"]                   = "Beep"

;  Initialize parameters describing the calculator and its keys

surface_color = 47  ;white on green
display_color = 15  ;white on black
key_color     = 48  ;black on cyan
depress_color = 94  ;yellow on magenta

surface_row = 2     ;where to put the calculator on the screen
surface_col = 10

display_height = 1  ;dimensions of the numeric display
display_width  = 20

n_key_rows = 4      ;# of rows and columns of calculator keys
n_key_cols = 5
   
key_height = 1      ;dimensions of a calculator key
key_width  = 5

label_row = int((key_height-1)/2)

spacing_height = 1  ;spacing between calculator keys
spacing_width  = 2

surface_height = display_height 
                 + (n_key_rows * (key_height + spacing_height))
                 + 2 * spacing_height + 2
surface_width  = (n_key_cols * (key_width + spacing_width))
                 + spacing_width + 2

;  Set up the pull-down menu for this application

ShowPullDown
   "Numbers":     "Select a Number":                         "Numbers"
      SubMenu
      "1":        "One":                                     "1",
      "2":        "Two":                                     "2",
      "3":        "Three":                                   "3",
      "4":        "Four":                                    "4",
      "5":        "Five":                                    "5",
      "6":        "Six":                                     "6",      
      "7":        "Seven":                                   "7",      
      "8":        "Eight":                                   "8",      
      "9":        "Nine":                                    "9",      
      "0":        "Ten":                                     "0"
      EndSubMenu,
   "Arithmetic":  "Select an Arithmetic Operator":           "Arithmetic"
      SubMenu
      "+":        "Plus":                                    "+",
      "-":        "Minus":                                   "-",
      "x":        "Times":                                   "X",
      "/":        "Divided By":                              "/",
      Separator,
      "=":        "Equals":                                  "="
      EndSubMenu,
   "Clear":       "Clear the Calculator Display":            "Clear"
      SubMenu
      "Entry":    "Clear the Value Being Entered":           "C",
      "All":      "Reset the Calculator":                    "ClearAll"
      EndSubMenu,
;  "EventTrace":  "Turn the Event Tracing On and Off":       "EventTrace"
;     SubMenu
;     "BeginTrace":  "Turn the Event Monitor On":      "EventTraceOn",
;     "EndTrace":    "Turn the Event Monitor Off":     "EventTraceOff",
;     "Configure":   "Review and Change Parameters":   "EventTraceConfig"
;     EndSubMenu,
   "Quit":        "All Done":                                "Quit"
EndMenu
   
EndProc

WriteLib out_lib Ten_Key_Init
Release Procs Ten_Key_Init

;  ------------------------------------------------------------------

Proc Create_Calc_Surface(is_floating)

;  Create the window which is the calculator surface.

DynArray surface_bag[]

surface_bag["Floating"] = is_floating
surface_bag["Title"]    = "Calculator"
surface_bag["Style"]    = surface_color

Window Create
   @ surface_row,surface_col
   Height surface_height Width surface_width
   Attributes surface_bag
   To surface_window

;  Set up to watch for the event which closes the surface window

do_acts["Message->Close->"+strval(surface_window)] = "Quit"

EndProc

WriteLib out_lib Create_Calc_Surface
Release Procs Create_Calc_Surface

;  ------------------------------------------------------------------

;  Process mouse and keystroke actions.

Proc Event_Loop()

depressed_key = ""                ;no depressed key yet
While True

;  Wait for some event from the user -- i.e., a mouse or keystroke

   GetEvent
      Mouse "Down","Up"
      Key "All"
      Message "All"
      To event_bag
      
   If is_event_trace then
      Event_Trace(event_bag)
   Endif

;  Conclude the pressing of a calculator key, pressed during the prior event

   If not isblank(depressed_key) then    ;we have a depressed key right now
      Key_Release()                      ;so process it and release it
      If do_act="Quit" then              ;the user pressed "Off"
         QuitLoop                        ;so quit
      Endif
   Endif

;  Convert an event into an action by processing the event using
;  the "dispatch" arrays

   do_default_act = "Beep"          ;the default action
   do_ix = event_bag["Type"]        ;start out with the event type
   While True                       ;several iterations needed to get action
      If isassigned(do_acts[do_ix]) then        ;a recognized event
         do_act = do_acts[do_ix]                ;do this event's action
         If isassigned(do_objs[do_ix]) then     ;the action has an object
            do_obj = do_objs[do_ix]             ;use that object
         Endif
         If isassigned(do_defs[do_ix]) then     ;there is a default action
            do_default_act = do_defs[do_ix]     ;remember the default
         Endif
      Else                                      ;an unknown event
         do_act = do_default_act                ;perform the default action
      Endif
      Switch
         Case do_act="AddEvElem":         ;action: add element to the index
            do_ix = do_ix + "->" + strval(event_bag[do_obj]) ;extend index
         Case do_act="AddEvKey":          ;action: add a key-code to index
            key_code = event_bag["KeyCode"]              ;the key-code
            If key_code>=32 and key_code<=126 then       ;it's typeable
               key_name = upper(chr(key_code))           ;convert to upper
               If key_name="*" then key_name="X" Endif   ;"*" is "X"
            Else                                         ;not typeable
               key_name = "Ascii "+strval(key_code)      ;use numeric ascii
            Endif
            do_ix = do_ix + "->" + key_name              ;extend the index
         Case do_act="AddWindow":         ;action: add window handle to index
            do_ix = do_ix + "->" + strval(getwindow())   ;extend the index
         Case do_act="Proc":              ;action: perform a procedure
            ExecProc do_obj               ;call the proc
            QuitLoop                      ;no further looping after proc
         Otherwise:                       ;some simple action
            QuitLoop                      ;no further looping
      EndSwitch
   EndWhile

;  Perform a simple action, such as beeping or quitting
   
   Switch
      Case do_act="Quit":        QuitLoop
      Case do_act="Beep":        Beep
      Case do_act="Execute":     ExecEvent event_bag
   EndSwitch
EndWhile

EndProc

WriteLib out_lib Event_Loop
Release Procs Event_Loop

;  ------------------------------------------------------------------

Proc Menu_Pick()

;  This procedure handles making a menu choice

;  Identify the key, then depress it and release it.

key_name = event_bag["MenuTag"]       ;the menu choice identifies the key
Key_Depress()                         ;push the key down
Key_Release()                         ;perform the function & release the key

EndProc

WriteLib out_lib Menu_Pick
Release Procs Menu_Pick

;  ------------------------------------------------------------------

Proc Key_Press()

;  This procedure handles the pressing of a key.

;  Depress and then release the key.

Key_Depress()                         ;push the key down
Key_Release()                         ;perform the function & release the key

EndProc

WriteLib out_lib Key_Press
Release Procs Key_Press

;  ------------------------------------------------------------------

Proc Key_Depress()

;  "Depress" a calculator key by changing its color

;  Set up the "depressed key," the one being operated on.

depressed_key = key_name         ;remember this key is depressed

;  Paint the key the special color it has when depressed

Key_Paint(depress_color)

EndProc

WriteLib out_lib Key_Depress
Release Procs Key_Depress

;  ------------------------------------------------------------------

Proc Key_Release()

;  "Release" a depressed calculator key and perform its function

;  Perform the key's function

Key_Function()

;  Paint the key its regular color

Key_Paint(key_color)

depressed_key = ""            ;it's not depressed any more

EndProc

WriteLib out_lib Key_Release
Release Procs Key_Release

;  ------------------------------------------------------------------

Proc Key_Paint(paint_color)

;  Paint a calculator key

;  Perform the procedure depending on whether this is program #1 or #2

ExecProc key_paint_proc

EndProc

WriteLib out_lib Key_Paint
Release Procs Key_Paint

;  ------------------------------------------------------------------

Proc Key_Function()

;  Perform the function assigned to a calculator key.

;  Deal with the key based on its type

do_act = ""                                 ;assume no further action

Switch
   Case depressed_key="O":                  ;the key is "Off"
      do_act = "Quit"                       ;prepare to quit
   Case depressed_key>="0" and depressed_key<="9":    ;a number key
      Key_Number()                          ;add the number to the display
   Case depressed_key=".":                  ;the decimal point key
      Key_Decimal()                         ;set up for decimal digits
   Case depressed_key="-" and is_new_num:   ;a negative sign
      Key_Neg()                             ;make the value negative
   Case depressed_key="C":                  ;the "Clear" key
      Key_Clear()                           ;perform the "Clear" function
   Otherwise:                               ;a calculation key
      Key_Arith()
EndSwitch

If depressed_key<>"C" then                  ;we didn't press the "Clear" key
   is_clearing = False                      ;so double-clear isn't possible
Endif

;  Display the new value

Display_Value()

EndProc

WriteLib out_lib Key_Function
Release Procs Key_Function

;  ------------------------------------------------------------------

Proc Key_Number()

;  Handle the pressing of a number key, 0 through 9.

Private
   d          ;the digit that has been pressed

;  If we're starting a new number after an arithmetic key,
;  switch the display from the old number to the new number

If is_new_num then                      ;the display is a completed number
   value_2 = 0                          ;clear the display
   decimal_factor = 1                   ;reset the decimal point
   n_decimal_2 = 0                      ;no decimal digits yet
   is_new_num = False                   ;not a new number any more
Endif

;  Determine the digit

d = numval(depressed_key)
If value_2<0 and not isblank(value_2) then
   d = -d
Endif

;  A digit to the left of the decimal point

If decimal_factor=1 then
   If value_2 >= 100000 then     ;we've reached the maximum
      Beep Return                ;can't add the new digit
   Endif
   value_2 = value_2 * 10 + d    ;add the new digit

;  A digit to the right of the decimal point

Else
   value_2 = value_2 + d*decimal_factor     ;add the new decimal digit
   decimal_factor = decimal_factor * 0.1    ;prepare for next decimal place
   n_decimal_2 = n_decimal_2 + 1            ;display one more decimal digit
Endif

;  Process a pending negative

If is_neg and value_2<>0 then
   value_2 = -value_2
   is_neg = False
Endif

EndProc

WriteLib out_lib Key_Number
Release Procs Key_Number

;  ------------------------------------------------------------------

Proc Key_Decimal()

;  Handle the pressing of the decimal point key.

If is_new_num then                      ;the display is a completed number
   value_2 = 0                          ;clear the display
   decimal_factor = 1                   ;reset the decimal point
   n_decimal_2 = 0                      ;no decimal places to display yet
   is_new_num = False                   ;not a new number any more
Endif

If decimal_factor<>1 then        ;we're already entering decimal digits
   Beep
Else
   decimal_factor = 0.1          ;set up to enter the first decimal digit
Endif

EndProc

WriteLib out_lib Key_Decimal
Release Procs Key_Decimal

;  ------------------------------------------------------------------

Proc Key_Neg()

;  Handle the pressing of the "-" key, where it means "make negative."

is_neg = not is_neg           ;switch the sign
If isblank(value_2) then      ;no value is being displayed
   value_2 = 0                ;so display zero
   n_decimal_2 = 0            ;no decimal places to display yet
Endif

EndProc

WriteLib out_lib Key_Neg
Release Procs Key_Neg

;  ------------------------------------------------------------------

Proc Key_Clear()

;  Handle the processing of the "Clear" key, to clear the current value.

;  If you press "Clear" twice in a row, the retained value and arithmetic
;  are also cleared.

If is_clearing then            ;this is the 2nd "Clear" in a row
   Double_Clear()              ;so clear all
   Return                      ;nothing else to do
Endif

;  Clear the displayed value

is_new_num = True              ;we're about to start typing a new number
value_2 = 0                    ;clear the value
n_decimal_2 = 0                ;no decimal places to display yet
is_neg  = False                ;no "-" pressed yet
is_clearing = True             ;we've pressed "Clear" once now

EndProc

WriteLib out_lib Key_Clear
Release Procs Key_Clear

;  ------------------------------------------------------------------

Proc Double_Clear()

;  Handle the double-pressing of the "Clear" key.
;  Clear out all stored values and arithmetic.

is_new_num = True              ;we're about to start typing a new number
value_1 = blanknum()           ;clear the non-displayed value
value_2 = blanknum()           ;clear the displayed value
is_neg  = False                ;no "-" pressed yet
pending_arith = ""             ;no pending arithmetic
is_clearing = False            ;we've pressed "Clear" twice now

Display_Value()                ;now clear the display too

EndProc

WriteLib out_lib Double_Clear
Release Procs Double_Clear

;  ------------------------------------------------------------------

Proc Key_Arith()

;  Handle the processing of an arithmetic function key:  + - x / =

;  Make sure there's something to process

If not isblank(value_2) then                   ;some value is keyed in

;  Perform any pending arithmetic from a prior function keypress

   Switch
      Case isblank(pending_arith):
         value_1 = value_2
      Case pending_arith="+":                   ;addition
         value_1 = value_1 + value_2
      Case pending_arith="-":                   ;subtraction
         value_1 = value_1 - value_2
      Case pending_arith="X":                   ;multiplication
         value_1 = value_1 * value_2
      Case pending_arith="/":                   ;division
         value_1 = value_1 / value_2
   EndSwitch
Endif

;  Preserve this function key to be performed later

If depressed_key = "=" then            ;it's the "=" function
   pending_arith = ""                  ;no future arithmetic
Else                                   ;it's + - * or /
   pending_arith = depressed_key       ;hold onto it for later
Endif

;  Initialize a new value for entry

is_new_num = True              ;we're about to start typing a new number
value_2 = blanknum()           ;clear the displayed value
is_neg  = False                ;no "-" pressed yet

EndProc

WriteLib out_lib Key_Arith
Release Procs Key_Arith

;  ------------------------------------------------------------------

Proc Display_Value()

;  Update the calculator display window

Private
   d,               ;the number of decimal digits to display
   f,               ;the control code string for the format() function
   need_neg,        ;whether a minus sign is needed
   s,               ;the value to display, as a text string
   v,               ;the value to display, as a number
   w                ;the display string width

;  Determine what value to display, and how many decimal digits to show

Switch
   Case isblank(value_2) and isblank(value_1):    v = 0
                                                  d = 0
   Case isblank(value_2):                         v = value_1
                                                  d = -1
   Otherwise:                                     v = value_2
                                                  d = n_decimal_2
EndSwitch

need_neg = is_neg
If v<0 then
   v = -v
   need_neg = not need_neg
Endif

;  Determine the number of decimal digits

If d<0 then
   s = strval(v)
   d = search(".",s)
   If d>0 then d=len(s)-d Endif
Endif

;  Convert the value into a text string

While True
   w = display_width - 1
   f = "W"+strval(w)+"."+strval(d)+",EC"
   s = format(f,v)
   If search("*",s)<=0 or d<=0 then
      QuitLoop
   Endif
   d = d-1      ;try again with fewer decimal digits
EndWhile

;  Insert the minus sign, if any

If need_neg then
   s = s + "-"
Else
   s = s + " "
Endif

;  Display the value string on the screen

ExecProc write_value_proc

EndProc

WriteLib out_lib Display_Value
Release Procs Display_Value

;  ------------------------------------------------------------------

