;  ------------------------------------------------------------------

Proc Closed Ten_Key_1()

;  A procedure using windows and mouse to simulate a 10-key calculator.
;  In this version, each key is a separate window.

Usevars
   autolib          ;the library name
   
Private
   decimal_factor,   ;the decimal-place factor, when we type in decimal digits
   depress_color,    ;color of a calculator key when depressed
   depressed_key,    ;the window handle for the depressed key
   display_bag,      ;dynarray describing the numeric display window
   display_color,    ;color of the numeric display surface
   display_height,   ;height of the numeric display window
   display_width,    ;width of the numeric display window
   display_window,   ;handle for calculator numeric display window
   do_act,           ;the action to perform in response to an event
   do_acts,          ;dynarray of actions to perform in response to events
   do_default_act,   ;the default action for an event, if do_act undetermined
   do_defs,          ;dynarray of default actions for unrecognized events
   do_ix,            ;the index built to convert an event to an action
   do_obj,           ;the object for "do_act" - e.g., a proc name
   do_objs,          ;dynarray of objects associated with "do_acts" actions
   ev_trace_params,  ;parameters controlling the event-trace window
   event_bag,        ;dynarray describing a mouse or keystroke event
   is_clearing,      ;whether we just pressed "Clear," & might press it again
   is_event_trace,   ;whether or not we are tracing events in a window
   is_neg,           ;whether we're typing in a negative number
   is_new_num,       ;whether we're about to type in a new value
   key_bag,          ;dynarray describing a calculator key
   key_code,         ;the Ascii code for a key which has been pressed
   key_color,        ;color of a calculator key
   key_height,       ;height (# of rows) of a calculator key
   key_label,        ;the label placed on a calculator key
   key_name,         ;the name of a calculator key
   key_names,        ;dynarray; for each calculator key, its name
   key_paint_proc,   ;procedure handling the painting of a calculator key
   key_width,        ;width (# of columns) of a calculator key
   key_window,       ;handle for calculator key window
   key_windows,      ;dynarray; for each calculator key, its window handle
   kr,kc,kn,         ;key row, key column, and key # values
   label_row,        ;row # for calculator key label
   n_decimal_2,      ;# of decimal digits for value_2   
   n_key_cols,       ;# of columns of calculator keys
   n_key_rows,       ;# of rows of calculator keys
   pending_arith,    ;the arithmetic operation waiting to be performed, if any
   r,c,              ;row and column values
   spacing_height,   ;distance (# of rows) between calculator key rows
   spacing_width,    ;distance (# of columns) between calculator key columns
   surface_bag,      ;dynarray describing calculator surface window
   surface_col,      ;column # for upper left corner of calculator surface
   surface_color,    ;color of the calculator surface
   surface_height,   ;height (# of rows) of the calculator surface
   surface_row,      ;row # for upper left corner of calculator surface
   surface_width,    ;width (# of columns) of the calculator surface
   surface_window,   ;handle for calculator surface window
   value_1,          ;the calculator value waiting for arithmetic
   value_2,          ;the calculator value being typed in right now
   write_value_proc  ;procedure handling writing to the calculator display

;  Initialize the display and various parameters

Ten_Key_Init()

;  Special event initialization

do_acts["Mouse->Left->False->Down"] = "Proc"             ;left mouse click
do_objs["Mouse->Left->False->Down"] = "Mouse_Depress_1"  ;call this proc
do_acts["Mouse->Right->True"]       = "Proc"             ;right double-click
do_objs["Mouse->Right->True"]       = "All_Fall_Down"    ;call this proc
do_acts["Key->F"]                   = "Proc"             ;press letter "F"
do_objs["Key->F"]                   = "All_Fall_Down"    ;call this proc

key_paint_proc   = "Key_Paint_1"
write_value_proc = "Write_Value_1"

;  Get event tracing ready, but don't turn it on yet

is_event_trace = False
;Event_Trace_Init()

;  Create the window which is the calculator surface.

Create_Calc_Surface(False)

;  Since the surface is a non-floating canvas,
;  we must use "Echo Normal" in order to see it.

Echo Normal

;  Paste the numeric display, as a window, on the calculator surface

r = surface_row + spacing_height + 1    ;row and column for display
c = surface_col + spacing_width  + 1

DynArray display_bag[]
display_bag["Style"]     = display_color
display_bag["HasFrame"]  = False
display_bag["HasShadow"] = False
display_bag["Title"]     = "Display"

Window Create
   Floating @ r,c
   Height display_height Width display_width
   Attributes display_bag
   To display_window

r = r + display_height + spacing_height    ;move to next screen row

;  Paste the number keys, as windows, on the calculator surface

DynArray key_bag[]
key_bag["Style"]     = key_color
key_bag["HasFrame"]  = False
key_bag["HasShadow"] = False

DynArray key_names[]
DynArray key_windows[]

kn = 0                                   ;initialize the key counter
For kr from 1 to n_key_rows              ;count key rows, top to bottom
   c = surface_col + spacing_width + 1   ;move to left screen column
   For kc from 1 to n_key_cols           ;count key columns, left to right
      kn = kn + 1                        ;one more key
   
;  Figure out which key to paste here

      Switch
         Case kr=1 and kc=1:   key_name="7"
         Case kr=1 and kc=2:   key_name="8"
         Case kr=1 and kc=3:   key_name="9"
         Case kr=2 and kc=1:   key_name="4"
         Case kr=2 and kc=2:   key_name="5"
         Case kr=2 and kc=3:   key_name="6"
         Case kr=3 and kc=1:   key_name="1"
         Case kr=3 and kc=2:   key_name="2"
         Case kr=3 and kc=3:   key_name="3"
         Case kr=4 and kc=1:   key_name="0"
         Case kr=4 and kc=2:   key_name="."
         Case kr=4 and kc=3:   key_name="="
         Case kr=1 and kc=4:   key_name="/"
         Case kr=2 and kc=4:   key_name="x"
         Case kr=3 and kc=4:   key_name="-"
         Case kr=4 and kc=4:   key_name="+"
         Case kr=1 and kc=5:   key_name="Off"
         Case kr=2 and kc=5:   key_name="C"
         Otherwise:            key_name=""
      EndSwitch

;  Paste this key onto the calculator surface

      If not isblank(key_name) then
         key_bag["Title"] = "Key "+key_name
         
         Window Create
            Floating @ r,c
            Height key_height Width key_width
            Attributes key_bag
            To key_window
            
         key_label = spaces(int((key_width-len(key_name))/2)) + key_name
         key_label = key_label + spaces(key_width - len(key_label))
         @ label_row,0 ?? key_label
         
;  Prepare parameters for locating and processing this key when it's pressed

         key_name = upper(substr(key_name,1,1))
         do_acts["Key->"+key_name] = "Proc"
         do_objs["Key->"+key_name] = "Key_Press"
         do_acts["Message->MenuSelect->"+key_name] = "Proc"
         do_objs["Message->MenuSelect->"+key_name] = "Menu_Pick"
         key_names[key_window] = key_name
         key_windows[key_name] = key_window
      Endif
      
      c = c + key_width + spacing_width    ;move to next screen column
   EndFor
   r = r + key_height + spacing_height     ;move to next screen row
EndFor   

;  Initialize the calculator values and display

Double_Clear()

;  Process mouse and keystroke actions.

Event_Loop()
   
;  All done.

Echo Off
AltSpace {Desktop} {Empty}         ;close all windows

EndProc

WriteLib out_lib Ten_Key_1
Release Procs Ten_Key_1

;  ------------------------------------------------------------------

Proc Mouse_Depress_1()

;  This procedure handles a mouse-down.
;  We're only interested in clicking on a calculator key with
;  the left button.

;  Determine whether the click occurred on a calculator key window

key_window = windowat(event_bag["Row"],event_bag["Col"])  ;identify window
If not isassigned(key_names[key_window]) then      ;not a calculator key
   do_act = "Execute"                              ;let Paradox handle it
   Return                                          ;ignore it
Endif
key_name = key_names[key_window]

;  Depress the calculator key

Key_Depress()

EndProc

WriteLib out_lib Mouse_Depress_1
Release Procs Mouse_Depress_1

;  ------------------------------------------------------------------

Proc Key_Paint_1()
   
;  Change the color of a calculator key by painting its window    

Private
   key_window         ;the window handle for the calculator key
   
;  Determine which window the key is, and then paint it "depressed."

key_window = key_windows[depressed_key]    ;determine its window
SetCanvas key_window
PaintCanvas Attribute paint_color All

EndProc

WriteLib out_lib Key_Paint_1
Release Procs Key_Paint_1

;  ------------------------------------------------------------------

Proc Write_Value_1()

;  Write the numeric value on the calculator by updating 
;  the display window

SetCanvas display_window
Clear
@ 0,0 ?? s

EndProc

WriteLib out_lib Write_Value_1
Release Procs Write_Value_1

;  ------------------------------------------------------------------

Proc All_Fall_Down()

;  Perform a special effect

Private
   cols,           ;array of window columns
   cvels,          ;column velocity of each window
   is_some_left,   ;are any windows left standing?
   max_row,        ;row # of lowest row a window can move to
   rows,           ;array of window rows
   rvels,          ;row velocity of each window
   sys_bag,        ;dynarray with sysinfo information
   wh,             ;current window handle
   whs,            ;array of window-handles
   i               ;temporary counter

;  Only do this if the calculator surface has been moved away from
;  all of its keys.

Window GetAttributes surface_window to surface_bag
If surface_bag["OriginCol"] + surface_width > surface_col - 5 and
   surface_bag["OriginCol"] < surface_col + surface_width + 5 then
      Return
Endif

;  Find out how large the screen is

SysInfo to sys_bag
max_row = sys_bag["ScreenHeight"] - key_height

;  Start with the display window and count the windows

Array whs[kn]
Array rows[kn]
Array cols[kn]
Array modes[kn]
Array rvels[kn]
Array cvels[kn]

Window Select display_window
For i from 1 to kn
   wh = getwindow()
   If wh=display_window and i>1 then
      kn = i-1
      QuitLoop 
   Endif
   Window GetAttributes wh to key_bag
   whs[i]  = wh
   rows[i] = key_bag["OriginRow"]
   cols[i] = key_bag["OriginCol"]
   modes[i] = 0
   WinNext
EndFor

;  Let 'em fall

is_some_left = True
While is_some_left
   is_some_left = False
   For i from 1 to kn
      If modes[i] >=0 then
         is_some_left = True
         Switch
            Case modes[i]=0:
               If rows[i] < max_row then
                  rows[i] = rows[i] + 1
               Else
                  modes[i] = 1
                  If i=1 then
                     rvels[i] = 3
                     cvels[i] = 1
                  Else
                     rvels[i] = int(rand()*5)
                     cvels[i] = int(rand()*7-3.5)
                  Endif
               Endif
            Case modes[i]=1:
               If rvels[i]<=0 then
                  modes[i] = -1
               Else
                  If rows[i] > max_row - rvels[i] then
                     rows[i] = rows[i] - 1
                     cols[i] = cols[i] + cvels[i]
                  Else
                     modes[i]=2
                  Endif
               Endif
            Case modes[i]=2:
               If rows[i] < max_row then
                  rows[i] = rows[i] + 1
                  cols[i] = cols[i] + cvels[i]
               Else
                  rvels[i] = rvels[i] - 1
                  modes[i] = 1
               Endif
         EndSwitch
         Window Move whs[i] to rows[i],cols[i]
      Endif
   EndFor
EndWhile

EndProc

WriteLib out_lib All_Fall_Down
Release Procs All_Fall_Down

;  ------------------------------------------------------------------
