;  ------------------------------------------------------------------

Proc CopyToDynArray(D)

;  Transfer the contents of the current record into the dynamic array "D".

;  Note:  You must create the dynarray "D" before calling this procedure.

Private
   F,          ;fixed array used to temporarily store the record
   fn,         ;the name of the current field
   fn1,        ;the name of the first field
   is_pull,    ;whether it's a pull-down menu (or horizontal menu)
   n           ;field counter

;  Retrieve the record into a fixed array

CopyToArray F

;  Determine the record's field names by traversing the choices
;  in the "Image/Zoom/Field" menu.

{Image} {Zoom} {Field}        ;get to the menu with field names
fn1 = menuchoice()            ;the name of the first field

Right                         ;in the old UI, this moves to the next name
is_pull = (menuchoice()=fn1)  ;if no movement it's new UI or 1-field table
Left                          ;return to the first menu choice

fn = fn1                      ;get ready to process the first field
n = 2                         ;initialize fixed-array index
While True                    ;do each field
   D[fn] = F[n]               ;transfer from fixed array to dynarray
   If is_pull then            ;it's a pull-down menu
      Down                    ;move to the next menu choice
      If menuchoice()=fn then QuitLoop Endif    ;we didn't move; all done
   Else                       ;it's a horizontal menu
      Right                   ;move to the next menu choice
      If menuchoice()=fn1 then QuitLoop Endif   ;back to beginning; all done
   Endif
   fn = menuchoice()          ;get ready to process this field
   n = n + 1                  ;increment the fixed-array index
EndWhile

;  All done

Menu Esc                      ;get out of the menus

EndProc

;  ------------------------------------------------------------------

Proc CopyFromDynArray(D)

;  Transfer the contents of the dynamic array "D" into the current record.

Private
   F,          ;fixed array used to temporarily store the record
   fn,         ;the name of the current field
   n,          ;field counter
   t           ;table name

;  Create a fixed-length array to store the record

t = table()             ;the name of the current table
CopyToArray F           ;this method of creating the fixed array avoids a
                        ;   run error if the dynarray lacks any fields

;  For each element in the dynamic array, if it names a field in the table,
;  transfer it's value to the fixed array

ForEach fn in D                     ;do each dynamic array element
   n = fieldno(fn,t)                ;determine the field number
   If n<>"Error" then               ;it is a valid field name
      F[n+1] = D[fn]                ;transfer the name
   Endif
EndForEach

;  Copy the record from the fixed array

CopyFromArray F

EndProc

;  ------------------------------------------------------------------

