/***
* Events.prg
*
* Simple event manager
*/

#include "Mouse.ch"
#include "Events.ch"

// Internal state of event manager
#define EVS_NONE     0     // No click yet
#define EVS_1_DOWN   1     // Button down
#define EVS_1_CLICK  2     // Button up again, so had one click
#define EVS_2_DOWN   3     // Button down again within timeout period

FUNCTION SetDblClick(nTime)

STATIC nDblClickTime := .3  // Make get / set function
LOCAL nOldTime := nDblClickTime

  IF !(nTime == NIL)
    nDblClickTime := nTime
  ENDIF

RETURN nOldTime


/*
// This version does not register a double click as a single click first
FUNCTION WaitEvent

LOCAL aEvent := {EV_NONE, NIL, NIL}
LOCAL nKey
LOCAL aMouseInfo := Array(M_SIZE_ARRAY)
LOCAL nEventState := EVS_NONE
LOCAL nStartClick
LOCAL nLastSglClick

  DO WHILE aEvent[EV_TYPE] == EV_NONE
    IF (nKey := InKey()) > 0
      aEvent[EV_KEY_VAL] := nKey
      aEvent[EV_TYPE] := EV_KEY
    ELSE
      MouseRead(aMouseInfo)
      DO CASE
        CASE nEventState == EVS_NONE
          IF MOUSE_DOWN(aMouseInfo)
            nEventState    := EVS_1_DOWN
            nStartClick := Seconds()
            aEvent[EV_ROW] := aMouseInfo[M_ROW]
            aEvent[EV_COL] := aMouseInfo[M_COL]
          ENDIF

        CASE nEventState == EVS_1_DOWN
          IF (aMouseInfo[M_ROW] != aEvent[EV_ROW]  .OR. ;
              aMouseInfo[M_COL] != aEvent[EV_COL]) .OR. ;
             Seconds() - nStartDblClick >= SetClick()
            nEventState := EVS_NONE
          ELSEIF !MOUSE_DOWN(aMouseInfo)
            nEventState := EVS_1_CLICK
          ENDIF

        CASE nEventState == EVS_1_CLICK
          IF (aMouseInfo[M_ROW] != aEvent[EV_ROW]  .OR. ;
              aMouseInfo[M_COL] != aEvent[EV_COL]) .OR. ;
             Seconds() - nStartDblClick >= SetDblClick()
            aEvent[EV_TYPE] := EV_SGL_CLICK
          ELSEIF MOUSE_DOWN(aMouseInfo)
            nEventState := EVS_2_DOWN
          ENDIF

        CASE nEventState == EVS_2_DOWN
          IF (aMouseInfo[M_ROW] != aEvent[EV_ROW]  .OR. ;
              aMouseInfo[M_COL] != aEvent[EV_COL]) .OR. ;
             Seconds() - nStartDblClick >= SetDblClick()
            aEvent[EV_TYPE] := EV_SGL_CLICK
          ELSEIF !MOUSE_DOWN(aMouseInfo)
            aEvent[EV_TYPE] := EV_DBL_CLICK
          ENDIF
      ENDCASE
    ENDIF
  ENDDO

RETURN aEvent
*/


// This version registers both a single and a double click

FUNCTION WaitEvent

LOCAL aEvent := {EV_NONE, NIL, NIL}
LOCAL nKey
LOCAL aMouseInfo := Array(M_SIZE_ARRAY)
LOCAL nEventState := EVS_NONE

STATIC nLastClick := 0
STATIC nSaveRow, nSaveCol

  DO WHILE aEvent[EV_TYPE] == EV_NONE
    IF (nKey := InKey()) > 0
      // This is easy - event type EV_KEY, and put key into EV_KEY_VAL
      aEvent[EV_KEY_VAL] := nKey
      aEvent[EV_TYPE] := EV_KEY
    ELSE
      MouseRead(aMouseInfo)
      DO CASE
        CASE nEventState == EVS_NONE
          IF MOUSE_DOWN(aMouseInfo)
            // Previously no state, now mouse down
            // Set state variable and record row and col position
            nEventState    := EVS_1_DOWN
            aEvent[EV_ROW] := aMouseInfo[M_ROW]
            aEvent[EV_COL] := aMouseInfo[M_COL]
          ENDIF

        CASE nEventState == EVS_1_DOWN
          IF (aMouseInfo[M_ROW] != aEvent[EV_ROW]  .OR. ;
              aMouseInfo[M_COL] != aEvent[EV_COL])
            // Mouse was down, but now it's moved. Switch
            // back to no state
            nEventState := EVS_NONE
          ELSEIF !MOUSE_DOWN(aMouseInfo)
            // Mouse was down, now it's up again
            // Record either single click or double click
            IF Seconds() - nLastClick > SetDblClick()
              // Not within SetDblClick() time
              // register single click and record row/col position and
              // current time for potential future double click
              aEvent[EV_TYPE] := EV_SGL_CLICK
              nSaveRow := aEvent[EV_ROW]
              nSaveCol := aEvent[EV_COL]
              nLastClick := Seconds()
            ELSEIF nSaveRow == aEvent[EV_ROW] .AND. ;
                   nSaveCol == aEvent[EV_COL]
              // At same position, so register double click
              aEvent[EV_TYPE] := EV_DBL_CLICK
              nLastClick := 0
            ENDIF
          ENDIF
      ENDCASE
    ENDIF
  ENDDO

RETURN aEvent
