; Mouse.asm
PUBLIC MouseInit, MouseOn, MouseOff, SetMouse, MouseRead

EXTRN  __retl    : FAR
EXTRN  __storni  : FAR
EXTRN  __storl   : FAR
EXTRN  __parinfo : FAR
EXTRN  __parl    : FAR

M_RESET        EQU  0
M_SHOW         EQU  1
M_REMOVE       EQU  2
M_READ         EQU  3
; M_CURS_WIDTH   EQU  8
; M_CURS_HEIGHT  EQU  8

FALSE          EQU  0
TRUE           EQU  1

MOUSE_INT      EQU  33H

DGROUP GROUP   data

data   SEGMENT PUBLIC  'DATA'

  lMouseVis DW FALSE       ; Off by default

data   ENDS


code   SEGMENT 'CODE'
             ASSUME cs:code, ds:DGROUP

MouseInit  PROC    FAR

             push    ds
             push    es
             push    si
             push    di

             mov     ax, M_RESET
             int     MOUSE_INT

             ; in preparation for call to _retl
             push    ax

             ; bx has number of buttons. We must pass this back to
             ; the parameter Clipper passed by reference
             ; bx into parameter 1 - must push in reverse order
             mov     ax, 1

             push    ax      ; into parameter number 1
             push    bx
             call    __storni
             add     sp, 4

             ; return ax we previously pushed
             call    __retl
             add     sp, 2

             ; Now reset lMouseVis flag to FALSE
             xor     ax, ax          ; mov ax, FALSE
             mov     lMouseVis, ax

             pop     di              ; Restore registers
             pop     si
             pop     es
             pop     ds

             ret
MouseInit    ENDP                    ; End of routine


MouseOn     PROC FAR

             push    ds
             push    es
             push    si
             push    di

             mov     ax, M_SHOW
             int     MOUSE_INT

             ; Now reset lMouseVis flag to TRUE
             mov     ax, TRUE
             mov     lMouseVis, ax

             pop     di              ; Restore registers
             pop     si
             pop     es
             pop     ds

             ret

MouseOn     ENDP


MouseOff   PROC FAR

             push    ds
             push    es
             push    si
             push    di

             mov     ax, M_REMOVE
             int     MOUSE_INT

             ; Now reset lMouseVis flag to FALSE
             xor     ax, ax          ; mov ax, FALSE
             mov     lMouseVis, ax

             pop     di              ; Restore registers
             pop     si
             pop     es
             pop     ds

             ret

MouseOff    ENDP


MouseRead  PROC FAR

             push    ds
             push    es
             push    si
             push    di

             mov     ax, M_READ
             int     MOUSE_INT

             push    cx              ; column
             push    dx              ; row
             push    bx              ; save button press info

set_left_down:
             ; in preparation for parameter element # 1, param # 1
             mov     ax, 1
             push    ax              ; array index
             push    ax              ; param number

             and     bx, 1
             push    bx              ; 1 if left down, 0 otherwise
             call    __storl
             add     sp, 6

set_right_down:
             pop     bx              ; restore button press info

             ; in preparation for parameter # 1, array index 2
             mov     ax, 2
             push    ax              ; array index 2
             dec     ax
             push    ax              ; param 1
             shr     bx, 1
             and     bx, 1
             push    bx

             call    __storl
             add     sp, 6

set_row:
             pop     dx              ; row

             ; Divide it by 8
             shr     dx, 1
             shr     dx, 1
             shr     dx, 1

             mov     ax, 3           ; array element # 3
             push    ax

             dec     ax
             dec     ax
             push    ax              ; param 1

             push    dx              ; row
             call    __storni
             add     sp,  6

set_col:
             pop     cx              ; column

             ; Divide it by 8
             shr     cx, 1
             shr     cx, 1
             shr     cx, 1

             mov     ax, 4           ; array element 4
             push    ax

             mov     ax, 1
             push    ax              ; param # 1
             push    cx              ; col
             call    __storni
             add     sp, 6

             pop     di              ; Restore registers
             pop     si
             pop     es
             pop     ds

             ret

MouseRead   ENDP


; Get / Set function for lMouseVis

SetMouse   PROC FAR

             push    ds
             push    es
             push    si
             push    di

             ; First return value to Clipper via _retl
             mov     ax, lMouseVis
             push    ax
             call    __retl
             add     sp, 2

             ; Now check whether a parameter was passed
             xor     ax, ax          ; short cut to make it 0
             push    ax
             call    __parinfo       ; pass parinfo 0 and it
                                     ; returns # params in ax
             add     sp, 2

             cmp     ax, 0
             jz      param_not_passed

             mov     ax, 1
             push    ax
             call    __parl
             add     sp, 2

             ; __parl conveniently returns result in ax
             mov     lMouseVis, ax

             ; If ax true, turn mouse on, otherwise off
             cmp     ax, FALSE
             jz      turn_mouse_off
turn_mouse_on:
             mov     ax, M_SHOW
             jmp short mouse_on_off
turn_mouse_off:
             mov     ax, M_REMOVE
mouse_on_off:
             int     MOUSE_INT
param_not_passed:
             pop     di              ; Restore registers
             pop     si
             pop     es
             pop     ds

             ret

SetMouse    ENDP

code         ENDS                    ; End of code segment
             END

